package webdb.sample.logic.impl;

import org.apache.commons.lang.RandomStringUtils;

import webdb.sample.dao.UserDao;
import webdb.sample.entity.User;
import webdb.sample.logic.DuplicatedEmailException;
import webdb.sample.logic.UserLogic;
import webdb.sample.support.Mailer;

public class UserLogicImpl implements UserLogic {

    private UserDao userDao;

    private Mailer mailer;

    public void registerNewUser(User user) throws DuplicatedEmailException {
        validateUniqueEmail(user.getEmail());
        String randomKey = generateRandomKey();
        user.setRandomKey(randomKey);
        userDao.insert(user);
        mailer.sendConfirmationMail(user);
    }

    /**
     * システムで一意となるランダムな文字列を生成します。
     * ここでは一意性は省略して24文字のランダム文字列が生成されます。
     */
    private String generateRandomKey() {
        return RandomStringUtils.randomAlphanumeric(24);
    }

    /**
     * 指定されたメールアドレスが登録されていないことを確認します。 
     * @param email 登録をチェックするメールアドレス
     * @throws DuplicatedEmailException メールアドレスが既に登録されている場合
     */
    private void validateUniqueEmail(String email)
            throws DuplicatedEmailException {
        if (userDao.getUserByEmail(email) != null) {
            throw new DuplicatedEmailException();
        }
    }

    public User approveRegistration(String key) {
        User user = userDao.getUserByRandomKey(key);
        if (user != null && !user.isValid()) {
            user.setValid(true);
            userDao.update(user);
            mailer.sendCompletionMail(user);
            return user;
        }
        return null;
    }

    public void setMailer(Mailer mailer) {
        this.mailer = mailer;
    }

    public void setUserDao(UserDao userDao) {
        this.userDao = userDao;
    }

}
